<?php

abstract class Downloader implements JsonSerializable
{
    public $url = null;
    public $title = null;
    public $thumbnail = null;
    public $thumbnailHotlinkProtection = false;
    public $duration = null;
    public $medias = [];
    public $source = null;
    public $sid = null;
    public ?Proxy $proxy = null;

    public abstract function fetch($videoUrl);

    public function qualityExists($quality)
    {
        foreach ($this->medias as $media) {
            if ($media->quality == $quality) {
                return true;
            }
        }
        return false;
    }

    public function mediaDetails()
    {
        if (!empty($this->duration)) {
            $this->duration = Helpers::formatSeconds((int)$this->duration);
        }
        if (empty($this->thumbnail)) {
            $this->thumbnail = 'https://cdn.nicheoffice.web.tr/image/9QMPjvQVBH.svg';
        }
        foreach ($this->medias as $media) {
            $media->getMediaInfo();
        }
    }

    public function saveToFile($clientIp)
    {
        $newRow = [
            'url' => $this->url,
            'title' => $this->title,
            'thumbnail' => $this->thumbnail,
            'source' => $this->source,
            'clientIp' => $clientIp
        ];
        $filePath = __DIR__ . '/../cache/latest-downloads.txt';
        if (!file_exists($filePath)) {
            $file = fopen($filePath, 'w');
            fwrite($file, '');
            fclose($file);
        }
        $newRowString = json_encode($newRow);
        $file = file_get_contents($filePath);
        $content = $newRowString . "\n" . $file;
        file_put_contents($filePath, $content);
    }

    public function saveToDatabase($clientIp)
    {
        $latestDownloads = get_option('aiodl_latest_downloads');
        $latestDownloads = json_decode($latestDownloads, true);
        if (!is_array($latestDownloads)) {
            $latestDownloads = array();
        }
        $limit = (int)get_option('aiodl_latest_downloads_count');
        array_unshift($latestDownloads, [
            'url' => $this->url,
            'title' => $this->title,
            'thumbnail' => $this->thumbnail,
            'source' => $this->source,
            'clientIp' => $clientIp
        ]);
        $count = count($latestDownloads);
        while ($count > $limit) {
            if (gettype($latestDownloads) === 'array') {
                $latestDownloads = array_pop($latestDownloads);
            }
            $count--;
        }
        update_option('aiodl_latest_downloads', json_encode($latestDownloads));
    }

    #[\ReturnTypeWillChange]
    public function jsonSerialize()
    {
        return [
            'url' => $this->url,
            'title' => $this->title,
            'thumbnail' => $this->thumbnail,
            'duration' => $this->duration,
            'source' => $this->source,
            'medias' => $this->medias,
            'sid' => $this->sid
        ];
    }

    public function isValid()
    {
        return $this->title != '' && isset($this->medias[0]->url) != '';
    }

    public function saveThumbnail()
    {
        if ($this->thumbnailHotlinkProtection && !empty($this->thumbnail)) {
            $this->thumbnail = Helpers::base64Image($this->thumbnail);
        }
    }

    public function encodeThumbnail()
    {
        if (!empty($this->thumbnail)) {
            $this->thumbnailHotlinkProtection = true;
            $this->saveThumbnail();
        }
    }

    public function setProxy(Proxy $proxy)
    {
        $this->proxy = $proxy;
    }
}