<?php

class Douyin extends Downloader
{
    private $userAgent = 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/117.0.0.0 Safari/537.36';
    private $baseUrl = 'https://www.douyin.com/aweme/v1/web/aweme/detail/?device_platform=webapp&aid=6383&channel=channel_pc_web&aweme_id={aweme_id}&pc_client_type=1&version_code=190500&version_name=19.5.0&cookie_enabled=true&screen_width=1920&screen_height=1080&browser_language=en-GB&browser_platform=MacIntel&browser_name=Chrome&browser_version=117.0.0.0&browser_online=true&engine_name=Blink&engine_version=117.0.0.0&os_name=Mac+OS&os_version=10.15.7&cpu_core_num=8&device_memory=8&platform=PC&downlink=10&effective_type=4g&round_trip_time=150&webid=7286877387768432182';

    public function fetch($videoUrl)
    {
        $http = new Http('https://douyin.wtf/api/hybrid/video_data?' . http_build_query(
                [
                    'url' => $videoUrl,
                ]
            ));
        $http->run();
        $data = json_decode($http->response, true);
        if (empty($data['data']['video'])) {
            return;
        }
        $data = $data['data'];
        $this->title = $data['desc'] ?? 'Douyin Video ' . rand(1, 100);
        $this->source = 'douyin';
        $this->thumbnail = $data['video']['cover']['url_list'][0];
        $this->duration = $data['video']['duration'];
        $this->medias[] = new Media($data['video']['download_addr']['url_list'][0], $data['video']['download_addr']['width'] . 'p', 'mp4', true, true);
        $this->encodeThumbnail();
    }

    public function fetch1($videoUrl)
    {
        $http = new Http('https://tenapi.cn/v2/video');
        $http->addCurlOption(CURLOPT_POSTFIELDS, [
            'url' => $videoUrl,
        ]);
        $http->addCurlOption(CURLOPT_CUSTOMREQUEST, 'POST');
        $http->run();
        $data = json_decode($http->response, true);
        if (empty($data['data']['url'])) {
            return;
        }
        $data = $data['data'];
        $this->title = !empty($data['title']) ? $data['title'] : 'Douyin Video' . rand(1, 100);
        $this->source = 'douyin';
        $this->thumbnail = $data['cover'];
        $this->medias[] = new Media($data['url'], 'hd', 'mp4', true, true);
    }

    public function fetchLegacy($videoUrl)
    {
        $videoId = $this->extractVideoId($videoUrl);
        $baseUrl = str_replace('{aweme_id}', $videoId, $this->baseUrl);
        $http = new Http('https://x-bogus-psi.vercel.app/');
        $http->addCurlOption(CURLOPT_CUSTOMREQUEST, 'POST');
        $http->addCurlOption(CURLOPT_POSTFIELDS, json_encode(['url' => $baseUrl, 'userAgent' => $this->userAgent]));
        $http->addHeader('Content-Type', 'application/json');
        $http->run();
        $data = json_decode($http->response, true);
        if (empty($data['data']['url'])) {
            return;
        }
        $data = $this->getVideoInfo($videoId, $data);
        $this->parseApiData($data['aweme_detail']);
    }

    private function parseApiData($data)
    {
        if (!empty($data['video'])) {
            $this->title = !empty($data['desc']) ? $data['desc'] : 'Douyin Video' . rand(1, 100);
            $this->source = 'douyin';
            $this->thumbnail = $data['video']['cover']['url_list'][0];
            $this->duration = (int)($data['video']['duration'] / 1000);
            $media = new Media($data['video']['play_addr']['url_list'][0], 'watermark', 'mp4', true, true);
            $media->size = $data['video']['play_addr']['data_size'];
            $this->medias[] = $media;
            $nwm = str_replace('playwm', 'play', $data['video']['play_addr']['url_list'][0]);
            $media = new Media($nwm, 'hd', 'mp4', true, true);
            $media->size = $data['video']['play_addr']['data_size'];
            $this->medias[] = $media;
        }
        if (!empty($data['music']['play_url']['url_list'])) {
            $media = new Media($data['music']['play_url']['url_list'][0], '128kbps', 'mp3', false, true);
            $media->size = $this->duration * 128 * 1000 / 8;
            $this->medias[] = $media;
        }
    }

    private function extractVideoId($videoUrl)
    {
        preg_match('/video\/(\d+)|modal_id=(\d+)|note\/(\d+)/', $videoUrl, $matches);
        if (!empty($matches[1])) {
            return $matches[1];
        }
        if (!empty($matches[2])) {
            return $matches[2];
        }
        if (!empty($matches[3])) {
            return $matches[3];
        }
        $http = new Http($videoUrl);
        $url = $http->getLongUrl();
        $url = strtok($url, '?');
        $last_char = substr($url, -1);
        if ($last_char == "/") {
            $url = substr($url, 0, -1);
        }
        $arr = explode('/', $url);
        return end($arr);
    }

    private function getVideoInfo($videoId, $data)
    {
        $http = new Http($data['data']['url']);
        $http->addHeader('Referer', 'https://www.douyin.com/video/' . $videoId);
        $http->addHeader('User-Agent', $this->userAgent);
        $http->addHeader('Cookie', 'ttwid:' . $data['data']['ttwid'] . ';');
        $http->addCurlOption(CURLOPT_TIMEOUT, 3);
        $http->run();
        return json_decode($http->response, true);
    }
}