<?php

class Pinterest extends Downloader
{
    public function fetch($videoUrl)
    {
        $parsedUrl = parse_url($videoUrl);
        $shortUrl = null;
        if ($parsedUrl['host'] == 'pin.it') {
            $shortUrl = $videoUrl;
            $http = new Http($videoUrl);
            $originalUrl = $http->getLongUrl();
            if (isset($originalUrl) != '') {
                $videoUrl = strtok($originalUrl, '?');
            }
        }
        preg_match('/\d{2,32}/', $videoUrl, $matches);
        $videoId = null;
        if (!empty($matches)) {
            $videoId = $matches[0];
        } else if ($shortUrl !== null) {
            $videoUrl = $this->convertShortUrl($shortUrl);
            $this->fetch($videoUrl);
        }
        if (empty($videoId)) {
            return;
        }
        $this->source = 'pinterest';
        $http = new Http('https://api.aiodown.com/v1/pinterest/pin/' . $videoId);
        $http->run();
        $data = json_decode($http->response, true);
        $data = $data['data'][0] ?? null;
        if (!empty($data['rich_metadata'])) {
            $this->title = $data['rich_metadata']['title'];
            if (empty($this->title)) {
                $this->title = $data['rich_metadata']['article']['name'] ?? 'Pinterest Video' . rand(1, 100);
            }
        }
        if (!empty($data['images'])) {
            $image = end($data['images']);
            $this->thumbnail = $image['url'];
            $this->medias[] = new Media($image['url'], min($image['height'], $image['width']) . 'p', 'jpg', false, false);
        }
        if (!empty($data['story_pin_data']) && empty($data['videos'])) {
            $data['videos']['video_list'] = $data['story_pin_data']['pages'][0]['blocks'][0]['video']['video_list'];
        }
        if (!empty($data['videos'])) {
            $this->thumbnail = $this->extractThumbnail($data);
            $this->duration = $this->extractDuration($data);
            foreach ($data['videos']['video_list'] as $stream) {
                $ext = pathinfo(parse_url($stream['url'])['path'], PATHINFO_EXTENSION);
                if ($ext != 'm3u8') {
                    $this->medias[] = new Media($stream['url'], min($stream['height'], $stream['width']) . 'p', $ext, true, true);
                }
            }
        }
        if ($this->title === null) {
            $this->title = 'Pinterest Video ' . rand(1, 100);
        }
    }

    private function convertShortUrl($shortUrl)
    {
        $parsedUrl = parse_url($shortUrl, PHP_URL_PATH);
        $realPath = str_replace('/', '', $parsedUrl);
        $redirectPage = 'https://api.pinterest.com/url_shortener/' . $realPath . '/redirect/';
        $http = new Http($redirectPage);
        return $http->getLongUrl();
    }

    private function extractThumbnail($data)
    {
        $key = key(array_slice($data['images'], -1, 1, true));
        return $data['images'][$key]['url'];
    }

    private function extractDuration($data)
    {
        $key = key(array_slice($data['videos']['video_list'], -1, 1, true));
        return (int)($data['videos']['video_list'][$key]['duration'] / 1000) ?? null;
    }
}