<?php

require_once __DIR__ . '/../FFmpeg.php';

class Reddit extends Downloader
{
    public function fetch($videoUrl)
    {
        preg_match('/reddit\.com\/(r|user)\/\w*\/comments\/\w*/', $videoUrl, $match);
        if (count($match) === 0) {
            return;
        }
        $baseUrl = 'https://www.' . $match[0];
        $http = new Http($baseUrl . '/.json');
        $http->run();
        $data = json_decode($http->response, true);
        if (empty($data[0]['data']['children'][0]['data'])) {
            return;
        }
        $data = $data[0]['data']['children'][0]['data'];
        if (!empty($data['url_overridden_by_dest'])) {
            if (substr($data['url_overridden_by_dest'], -4) === '.gif') {
                $this->parseGifData($data);
                return;
            }
            $host = parse_url($data['url'], PHP_URL_HOST);
            if ($host == 'youtube.com' || $host == 'www.youtube.com') {
                require_once __DIR__ . "/YouTube.php";
                $yt = new YouTube();
                $yt->fetch($data['url']);
                $this->title = $yt->title;
                $this->source = $yt->source;
                $this->thumbnail = $yt->thumbnail;
                $this->medias = $yt->medias;
                $this->duration = $yt->duration;
                return;
            }
        }
        if (!($data['is_video'] ?? false)) {
            return;
        }
        if (empty($data['secure_media'])) {
            return;
        }
        $this->title = $data['title'];
        $this->source = 'reddit';
        $this->thumbnail = $data['thumbnail'];
        if ($this->thumbnail == 'default') {
            $this->thumbnail = 'https://cdn.nicheoffice.web.tr/image/mZB7uSc3IT.png';
        }
        $this->duration = $data['secure_media']['reddit_video']['duration'];
        $playlistUrl = $data['secure_media']['reddit_video']['dash_url'];
        preg_match('/https?:\/\/v\.redd\.it\/(?P<id>[^\/?#&]+)/', $playlistUrl, $match);
        if (empty($match['id'])) {
            return;
        }
        $videoId = $match['id'];
        $http = new Http($playlistUrl);
        $http->run();
        $xmlPlaylist = $http->response;
        preg_match_all('/<BaseURL>(.*)<\/BaseURL>/', $xmlPlaylist, $medias);
        if ($medias[1] != '') {
            $medias = $medias[1];
            $audioUrl = '';
            $videos = [];
            $audios = [];
            foreach ($medias as $media) {
                preg_match('/DASH_(\d*|AUDIO_(\d*))\.mp4/', $media, $match);
                $isAudio = count($match) === 3;
                $quality = $isAudio ? $match[2] . 'kbps' : $match[1] ?? '720' . 'p';
                $mediaUrl = 'https://v.redd.it/' . $videoId . '/' . $media;
                if ($isAudio) {
                    $audioUrl = $mediaUrl;
                    $audios[$quality] = $mediaUrl;
                    continue;
                }
                $videos[$quality] = $mediaUrl;
            }
            foreach ($videos as $quality => $mediaUrl) {
                if (empty($audioUrl)) {
                    $this->medias[] = new Media($mediaUrl, $quality, 'mp4', true, true);
                    continue;
                }
                $ffmpegUrl = FFmpeg::combinedLink($mediaUrl, $audioUrl, $this->title . '.mp4');
                $media = new Media($ffmpegUrl, $quality, 'mp4', true, true);
                $media->requiresRendering = true;
                $http = new Http($mediaUrl);
                $media->size = $http->getFileSize();
                $this->medias[] = $media;
            }
            foreach ($audios as $quality => $mediaUrl) {
                $this->medias[] = new Media($mediaUrl, $quality, 'm4a', false, true);
            }
        }
    }

    private function parseGifData($data)
    {
        $this->title = $data['title'];
        $this->source = 'reddit';
        $this->thumbnail = $data['thumbnail'];
        if (empty($data['preview']['images'][0]['variants'])) {
            return;
        }
        $gifUrl = $data['preview']['images'][0]['variants']['gif']['source']['url'] ?? '';
        if ($gifUrl != '') {
            $this->medias[] = new Media($gifUrl, 'SD', 'gif', true, false);
        }
        $mp4Url = $data['preview']['images'][0]['variants']['mp4']['source']['url'] ?? '';
        if ($mp4Url != '') {
            $this->medias[] = new Media($mp4Url, 'SD', 'mp4', true, false);
        }
    }
}