<?php

class Tiktok extends Downloader
{
    private static $cookieFile = __DIR__ . '/../../cookies/tiktok-cookie.txt';

    public function fetch($videoUrl)
    {
        $host = parse_url($videoUrl, PHP_URL_HOST);
        if ($host == 'vm.tiktok.com') {
            $videoUrl = $this->getRedirectedUrl($videoUrl);
            $videoUrl = strtok($videoUrl, '?');
        }
        $videoPage = $this->getVideoPage($videoUrl);
        preg_match('/<script id="__UNIVERSAL_DATA_FOR_REHYDRATION__" type="application\/json">(.*?)<\/script>/', $videoPage, $matches);
        $data = $matches[1];
        $data = json_decode($data, true);
        if (empty($data['__DEFAULT_SCOPE__']['webapp.video-detail'])) {
            return;
        }
        $data = $data['__DEFAULT_SCOPE__']['webapp.video-detail']['itemInfo']['itemStruct'];
        $author = $data['author']['uniqueId'];
        $nickname = $data['author']['nickname'];
        $title = ($nickname ?: $author) . ' [' . rand(1000, 9999) . ']';
        $this->title = $data['desc'] ?? $title;
        $this->source = 'tiktok';
        $this->duration = $data['video']['duration'];
        $this->thumbnail = $data['video']['cover'];
        if (!empty($data['video']['playAddr'])) {
            $media = new Media($data['video']['playAddr'], 'hd', 'mp4', true, true);
            $media->size = self::getSize($data['video']['playAddr']);
            $this->medias[] = $media;
        }
        if (!empty($data['video']['downloadAddr'])) {
            $media = new Media($data['video']['downloadAddr'], 'watermark', 'mp4', true, true);
            $media->size = self::getSize($data['video']['playAddr']);
            $this->medias[] = $media;
        }
        /*
        if (!empty($data['music'])) {
            $this->medias[] = new Media($data['music']['playUrl'], '128kbps', 'mp3', false, true);
        }*/
        $this->encodeThumbnail();
    }

    private function getRedirectedUrl($longUrl)
    {
        $http = new Http($longUrl);
        $http->addHeader('User-Agent', 'facebookexternalhit/1.1');
        return $http->getLongUrl();
    }

    private function getHeaders()
    {
        $http = new Http('https://api.aiodown.com/v1/tiktok/headers');
        $http->run();
        return json_decode($http->response);
    }

    private function getVideoPage($url)
    {
        $headers = $this->getHeaders();
        if (empty($headers)) {
            return "";
        }
        $http = new Http($url);
        $http->addCurlOption(CURLOPT_HTTPHEADER, $headers);
        $http->addCurlOption(CURLOPT_COOKIEJAR, self::$cookieFile);
        $http->run();
        return $http->response;
    }


    private static function getSize($videoUrl)
    {
        $curl = curl_init();
        curl_setopt_array($curl, [
            CURLOPT_URL => $videoUrl,
            CURLOPT_ENCODING => "",
            CURLOPT_MAXREDIRS => 3,
            CURLOPT_TIMEOUT => 5,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_NOBODY => true,
            CURLOPT_HTTPHEADER => [
                "accept: */*",
                "accept-encoding: gzip, deflate",
                "connection: close",
                "host: v16-webapp-prime.tiktok.com",
                "referer: https://www.tiktok.com/",
                "user-agent: Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:88.0) Gecko/20100101 Firefox/88.0"
            ],
            CURLOPT_COOKIEFILE => self::$cookieFile,
        ]);
        curl_exec($curl);
        curl_error($curl);
        $size = -1;
        if (curl_errno($curl) == 0) {
            $size = curl_getinfo($curl, CURLINFO_CONTENT_LENGTH_DOWNLOAD);
        }
        curl_close($curl);
        return $size;
    }

    public static function forceDownload($url, $name, $extension, $size)
    {
        header("Pragma: public");
        header("Expires: 0");
        header("Cache-Control: must-revalidate, post-check=0, pre-check=0");
        header("Cache-Control: private", false);
        header('Content-Type: application/octet-stream');
        header('Content-Disposition: attachment; filename="' . Helpers::sanitizeFilename($name, $extension) . '"');
        header("Content-Transfer-Encoding: binary");
        header("Accept-Ranges: bytes");
        header("Content-Ranges: bytes");
        if ($size > 100) {
            header('Content-Length: ' . $size);
        } else {
            $size = self::getSize($url);
        }
        header('Connection: Close');
        @ini_set('max_execution_time', 0);
        @set_time_limit(0);
        if (ob_get_length() > 0) {
            ob_clean();
        }
        flush();
        // Activate flush
        if (function_exists('apache_setenv')) {
            apache_setenv('no-gzip', 1);
        }
        @ini_set('zlib.output_compression', false);
        ini_set('implicit_flush', true);
        ini_set('memory_limit', '256M');
        // CURL Process
        $ch = curl_init();
        $chunkEnd = $chunkSize = 1000000; // 1 MB in bytes
        $tries = $count = $chunkStart = 0;
        while ($size > $chunkStart) {
            curl_setopt($ch, CURLOPT_HEADER, 0);
            curl_setopt($ch, CURLOPT_URL, $url);
            curl_setopt($ch, CURLOPT_FOLLOWLOCATION, 1);
            curl_setopt($ch, CURLOPT_HTTPHEADER, [
                "accept: */*",
                "accept-encoding: gzip, deflate",
                "connection: close",
                "host: v16-webapp-prime.tiktok.com",
                "referer: https://www.tiktok.com/",
                "user-agent: Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:88.0) Gecko/20100101 Firefox/88.0"
            ]);
            curl_setopt($ch, CURLOPT_COOKIEFILE, self::$cookieFile);
            curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
            curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
            curl_setopt($ch, CURLOPT_BUFFERSIZE, $chunkSize);
            curl_setopt($ch, CURLOPT_TIMEOUT, 5);
            $output = curl_exec($ch);
            $curlInfo = curl_getinfo($ch);
            if ($curlInfo['http_code'] != "206" && $curlInfo['http_code'] != '403' && $tries < 10) {
                $tries++;
                continue;
            } else {
                if ($tries === 0 && $curlInfo['http_code'] == '403') {
                    Http::forceDownloadLegacy($url, $name, $extension, $size);
                    exit;
                }
                $tries = 0;
                echo $output;
                flush();
                ob_implicit_flush(true);
                if (ob_get_length() > 0) ob_end_flush();
            }
            $chunkStart += self::$chunkSize;
            $chunkStart += ($count == 0) ? 1 : 0;
            $chunkEnd += self::$chunkSize;
            $count++;
        }
        curl_close($ch);
        exit;
    }
}