<?php

class Twitch extends Downloader
{

    public function fetch($videoUrl)
    {
        preg_match('/videos\/(\d+)/m', $videoUrl, $matches);
        $clipName = $this->extractClipName($videoUrl);
        if ($clipName != '') {
            $this->fetchClip($videoUrl, $clipName);
            return;
        }
        if (empty($matches) || count($matches) != 2) {
            return;
        }
        $videoId = $matches[1];
        $http = new Http('https://api.aiodown.com/v1/twitch/queries');
        $http->run();
        $queries = json_decode($http->response, true);
        $videoQuery = str_replace('{{videoId}}', $videoId, $queries['videoMetadata']);
        $authQuery = str_replace('{{videoId}}', $videoId, $queries['accessToken']);
        $data = $this->apiRequest($videoQuery)[0]['data']['video'];
        $this->title = $data['title'] ?? 'Twitch Video';
        $this->source = 'twitch';
        $this->thumbnail = $data['previewThumbnailURL'] ?? 'https://static-cdn.jtvnw.net/ttv-static-metadata/twitch_logo3.jpg';
        $this->duration = $data['lengthSeconds'];
        $auth = $this->apiRequest($authQuery);
        $signature = $auth['data']['videoPlaybackAccessToken']['signature'];
        $expires = json_decode($auth['data']['videoPlaybackAccessToken']['value'], true)['expires'];
        $playlistUrl = str_replace('{{signature}}', $signature, $queries['playlistUrl']);
        $playlistUrl = str_replace('{{expires}}', $expires, $playlistUrl);
        $playlistUrl = str_replace('{{videoId}}', $videoId, $playlistUrl);
        $http = new Http($playlistUrl);
        $http->run();
        preg_match_all('/RESOLUTION=\d*x(\d*).*\s(.*)/m', $http->response, $qualities);
        if (!empty($qualities) && count($qualities) === 3) {
            for ($i = 0; $i < count($qualities); ++$i) {
                if (empty($qualities[2][$i])) {
                    continue;
                }
                $http = new Http($qualities[2][$i]);
                $http->run();
                preg_match_all('/(\d*.ts)/m', $http->response, $chunks);
                $chunks = $chunks[0];
                $baseUrl = dirname($qualities[2][$i]);
                for ($j = 0; $j < count($chunks); ++$j) {
                    $chunks[$j] = $baseUrl . '/' . $chunks[$j];
                }
                $chunkSize = new Http($chunks[0]);
                $chunkSize = $chunkSize->getFileSize();
                $chunkFile = new Cache('twitch-' . $videoId . '-' . $qualities[1][$i], 'json', json_encode($chunks));
                $media = new Media($chunkFile->url, $qualities[1][$i] . 'p', 'mp4', true, true);
                $media->size = $chunkSize * count($chunks);
                $media->chunked = true;
                $this->medias[] = $media;
            }
            usort($this->medias, array('Helpers', 'sortByQuality'));
        }
    }

    private function fetchClip($videoUrl, $clipName)
    {
        $this->title = 'Twitch Video Clip from ' . $this->extractPosterName($videoUrl);
        $this->source = 'twitch';
        $this->thumbnail = 'https://static-cdn.jtvnw.net/ttv-static-metadata/twitch_logo3.jpg';
        $data = $this->apiRequest($this->queryClip($clipName))[0];
        if (isset($data['data']['clip']['videoQualities']) != '') {
            $signature = '?' . http_build_query([
                    'sig' => $data['data']['clip']['playbackAccessToken']['signature'],
                    'token' => $data['data']['clip']['playbackAccessToken']['value'],
                ]);
            foreach ($data['data']['clip']['videoQualities'] as $video) {
                $this->medias[] = new Media($video['sourceURL'] . $signature, $video['quality'] . 'p', 'mp4', true, true);
            }
            usort($this->medias, array('Helpers', 'sortByQuality'));
        }
    }

    private function extractClipName($videoUrl)
    {
        $parsedUrl = parse_url($videoUrl);
        $path = explode('/', $parsedUrl['path']);
        if (count($path) == 2) {
            return $path[1];
        } else if ($path[2] == 'clip' && isset($path[3]) != '') {
            return $path[3];
        } else {
            return false;
        }
    }

    private function extractPosterName($videoUrl)
    {
        $parsedUrl = parse_url($videoUrl);
        $path = explode('/', $parsedUrl['path']);
        if (count($path) == 2) {
            return $path[1];
        } else if ($path[2] == 'clip' && isset($path[3]) != '') {
            return $path[1];
        } else {
            return false;
        }
    }

    private function queryClip($clipName)
    {
        $operation = array(
            0 =>
                array(
                    'operationName' => 'VideoAccessToken_Clip',
                    'variables' =>
                        array(
                            'slug' => $clipName,
                        ),
                    'extensions' =>
                        array(
                            'persistedQuery' =>
                                array(
                                    'version' => 1,
                                    'sha256Hash' => '36b89d2507fce29e5ca551df756d27c1cfe079e2609642b4390aa4c35796eb11',
                                ),
                        ),
                ),
        );
        return json_encode($operation);
    }

    private function apiRequest($postData, $jsonDecode = true)
    {
        $http = new Http('https://gql.twitch.tv/gql');
        $http->addCurlOption(CURLOPT_CUSTOMREQUEST, 'POST');
        $http->addCurlOption(CURLOPT_POSTFIELDS, $postData);
        $http->addHeader('client-id', 'kimne78kx3ncx6brgo4mv6wki5h1ko');
        $http->addHeader('user-agent', 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/93.0.4556.0 Safari/537.36');
        $http->addHeader('sec-fetch-mode', 'navigate');
        $http->addHeader('Content-Type', 'application/json');
        $http->run();
        return $jsonDecode ? json_decode($http->response, true) : $http->response;
    }
}