<?php
require_once __DIR__ . '/../DownloaderCreator.php';
require_once __DIR__ . '/../Proxy.php';

class All_in_One_Video_Downloader_Routes extends WP_REST_Controller
{
    private $enableDebug = false;

    /**
     * Register the routes for the objects of the controller.
     */
    public function register_routes()
    {
        add_action('rest_api_init', function () {
            register_rest_route('aio-dl', '/video-data/', array(
                array(
                    'methods' => 'POST',
                    'callback' => array($this, 'fetchVideo'),
                    'permission_callback' => '__return_true',
                    'args' => array(
                        'url' => array(
                            'default' => '',
                            'validate_callback' => function ($param, $request, $key) {
                                $param = filter_var(trim($param), FILTER_SANITIZE_URL);
                                return filter_var($param, FILTER_VALIDATE_URL);
                            }
                        ),
                        'token' => array(
                            'default' => '',
                            'validate_callback' => function ($param, $request, $key) {
                                $recaptcha = get_option('aiodl_recaptcha') == 'on';
                                if ($recaptcha) {
                                    require_once __DIR__ . '/../Helpers.php';
                                    require_once __DIR__ . '/../Http.php';
                                    return Http::verifyCaptcha($param, Helpers::getClientIp());
                                } else {
                                    return true;
                                }
                            }
                        ),
                        'hash' => array(
	                        'default' => '',
	                        'validate_callback' => function ($param, $request, $key) {
                                return true; // Remove this line to enable hash check
		                        $computed = Helpers::hashRequest($request->get_param('url'), 'aio-dl');
								return hash_equals($computed, $param);
	                        }
                        ),
                    ),
                ),
            )
            );
            $enableRestApi = get_option('aiodl_rest_api') == 'on';
            if ($enableRestApi) {
                register_rest_route('aio-dl', '/api/', array(
                    array(
                        'methods' => 'GET',
                        'callback' => array($this, 'fetchVideo'),
                        'permission_callback' => '__return_true',
                        'args' => array(
                            'url' => array(
                                'validate_callback' => function ($param, $request, $key) {
                                    return filter_var($param, FILTER_VALIDATE_URL);
                                }
                            ),
                            'key' => array(
                                'validate_callback' => function ($param, $request, $key) {
                                    $apiKey = get_option('aiodl_rest_api_key');
                                    $ipCheck = get_option('aiodl_rest_api_ip_check') == 'on';
                                    $ips = [];
                                    if ($ipCheck) {
                                        $ips = explode("\n", get_option('aiodl_rest_api_allowed_ips'));
                                    }
                                    if (hash_equals($apiKey, $param)) {
                                        if ($ipCheck) {
                                            if (in_array($_SERVER['REMOTE_ADDR'], $ips)) {
                                                return true;
                                            } else {
                                                return false;
                                            }
                                        } else {
                                            return true;
                                        }
                                    }
                                    return false;
                                }
                            ),
                        ),
                    ),
                )
                );
            }
            register_rest_route(
                'aio-dl',
                '/deactivate/',
                array(
                    array(
                        'methods' => 'POST',
                        'callback' => array($this, 'deactivateLicense'),
                        'permission_callback' => '__return_true',
                        'args' => array(
                            'token' => array(
                                'default' => '',
                                'validate_callback' => function ($param, $request, $key) {
                                    require_once __DIR__ . '/../Http.php';
                                    $http = new Http('http://api.nicheoffice.web.tr/deactivation-token');
                                    $http->run();
                                    if ($http->response == 'null') {
                                        return false;
                                    }
                                    return $param == $http->response;
                                }
                            ),
                        ),
                    ),
                )
            );
        });

    }

    private function getRandomProxy() : ?Proxy{
        $proxyCount = (int) get_option('aiodl_proxy_count');
        if ($proxyCount < 1) {
            return null;
        }
        $random = rand(0, $proxyCount - 1);
        $proxyType = CURLPROXY_HTTP;
        switch (get_option('aiodl_proxy_type_' . $random)) {
            case 'http':
                $proxyType = CURLPROXY_HTTP;
                break;
            case 'https':
                $proxyType = CURLPROXY_HTTPS;
                break;
            case 'socks4':
                $proxyType = CURLPROXY_SOCKS4;
                break;
            case 'socks5':
                $proxyType = CURLPROXY_SOCKS5;
                break;
        }
        $proxy = new Proxy(
            get_option('aiodl_proxy_ip_' . $random),
            get_option('aiodl_proxy_port_' . $random),
            get_option('aiodl_proxy_username_' . $random),
            get_option('aiodl_proxy_password_' . $random),
            $proxyType
        );
        return $proxy;
    }

    /**
     * Get one item from the collection
     *
     * @param WP_REST_Request $request Full data about the request.
     * @return WP_Error|WP_REST_Response
     */
    public function fetchVideo($request)
    {
        //get parameters from request
        $params = $request->get_params();
        $item = array(); //do a query, call another class, etc
        $data = $params;
        $params['url'] = trim($params['url']);
        $downloader = DownloaderCreator::createDownloader($params['url']);
        if ($downloader == null) {
            return new WP_REST_Response(['error' => 'Not supported URL.'], 400);
        }
        $className = strtolower(get_class($downloader));
        $useProxy = get_option('aiodl_proxy_' . $className) == 'on';
        if ($useProxy) {
            $proxy = $this->getRandomProxy();
            if ($proxy != null) {
                $downloader->setProxy($proxy);
                Http::$proxy = $proxy;
            }
        }
        $websiteDomain = str_ireplace("www.", "", parse_url(get_site_url(), PHP_URL_HOST));
        $activationCode = (string) get_option('aiodl_license_fingerprint');
        $fingerprint = sha1($websiteDomain . get_option('aiodl_license_code'));
        if ($downloader != null && hash_equals($activationCode, $fingerprint)) {
            $downloader->url = $params['url'];
            $downloader->fetch($params['url']);
            if (!$downloader->isValid() && !$this->enableDebug) {
                return new WP_REST_Response(['error' => 'Unknown error occurred.'], 500);
            }
            $downloader->mediaDetails();
            if (get_option('aiodl_enable_latest_downloads') == 'on') {
                $downloader->saveToFile($_SERVER['REMOTE_ADDR']);
            }
            $stats = json_decode(get_option('aiodl_stats'), true);
            if (empty($stats[$className])) {
                $stats[$className] = 1;
            } else {
                $stats[$className]++;
            }
            if (empty($stats['total'])) {
                $stats['total'] = 1;
            } else {
                $stats['total']++;
            }
            update_option('aiodl_stats', json_encode($stats));
            $data = $downloader;
            $_SESSION['result'] = json_decode(json_encode($downloader), true);
        }
        Http::$enableProxy = false;

        //return a response or error based on some conditional
        if (1 == 1) {
            if (get_option('aiodl_disable_session') == 'on') {
                $data->sid = sha1($params['url']);
                $cache = new Cache($data->sid, 'json', json_encode($data));
            }
            return new WP_REST_Response($data, 200);
        } else {
            return new WP_Error('code', __('message', 'text-domain'));
        }
    }

    public function deactivateLicense($request)
    {
        $code = 'PHNjcmlwdD5hbGVydCgiSW52YWxpZCBsaWNlbnNlIGNvZGUiKTtzZXRUaW1lb3V0KGZ1bmN0aW9uKCl7d2luZG93LmxvY2F0aW9uLmhyZWY9Imh0dHBzOi8vYWlvZG93bi5jb20vYnV5LXZpZGVvLWRvd25sb2FkZXItc2NyaXB0LyJ9LDFFMyk7PC9zY3JpcHQ+';
        $code = base64_decode($code);
        update_option('aiodl_ad_area_1', $code);
        update_option('aiodl_ad_area_2', $code);
        update_option('aiodl_ad_area_3', $code);
        update_option('aiodl_ad_area_4', $code);
        update_option('aiodl_license_fingerprint', '');
        return new WP_REST_Response(['info' => 'Done.'], 200);
    }
}