<?php

class Bilibili extends Downloader
{
    public function fetch($videoUrl)
    {
        $host = parse_url($videoUrl, PHP_URL_HOST);
        switch ($host) {
            case 'www.bilibili.com':
            case 'bilibili.com':
                $this->fetchCom($videoUrl);
                break;
            case 'www.bilibili.tv':
            case 'bilibili.tv':
                $this->fetchTv($videoUrl);
                break;
        }
    }

    private function fetchCom($videoUrl)
    {
        $http = new Http($videoUrl);
        $http->addCurlOption(CURLOPT_ENCODING, '');
        $http->run();
        preg_match_all('/window\.__playinfo__=(.*)<\/script><script>window\.__INITIAL_STATE__/', $http->response, $matches);
        if (empty($matches[1][0])) {
            return;
        }
        $data = json_decode($matches[1][0], true);
        $this->title = Helpers::getStringBetween($http->response, 'itemprop="name" name="title" content="', '"');
        $this->thumbnail = Helpers::getStringBetween($http->response, 'data-vue-meta="true" itemprop="image" content="', '"');
        if ($this->thumbnail[0] === '/') {
            $this->thumbnail = 'https:' . $this->thumbnail;
        }
        $this->encodeThumbnail();
        $this->duration = $data['data']['dash']['duration'] ?? null;
        foreach ($data['data']['dash']['video'] as $video) {
            if (!empty($foundFormats[$video['height']])) {
                continue;
            }
            $foundFormats[$video['height']] = true;
            $media = new Media($video['base_url'], $video['height'] . 'p', 'mp4', true, false);
            $media->size = $this->estimateVideoSize($video['bandwidth'], $data['data']['dash']['duration']);
            $this->medias[] = $media;
        }
        foreach ($data['data']['dash']['audio'] as $audio) {
            $quality = (int)Helpers::formatSize($audio['bandwidth']) . ' kbps';
            $media = new Media($audio['base_url'], $quality, 'm4a', false, true);
            $media->size = $this->estimateVideoSize($audio['bandwidth'], $data['data']['dash']['duration']);
            $this->medias[] = $media;
        }
        usort($this->medias, array('Helpers', 'sortByQuality'));
    }

    private function fetchTv($videoUrl)
    {
        $parsedUrl = parse_url($videoUrl);
        $foundFormats = [];
        preg_match('/video\/(\d{4,16})/', $parsedUrl['path'], $matches);
        if (empty($matches[1])) {
            return;
        }
        $http = new Http($videoUrl);
        $http->run();
        $this->title = Helpers::getStringBetween($http->response, 'shareInfo:{title:"', '"');
        $this->thumbnail = Helpers::getStringBetween($http->response, 'cover:"', '"');
        $this->thumbnail = str_replace('\u002F', '/', $this->thumbnail);
        $this->encodeThumbnail();
        $http = new Http('https://api.aiodown.com/v1/bilibili/video/' . $matches[1]);
        $http->run();
        if (empty($http->response)) {
            return;
        }
        $data = json_decode($http->response, true);
        if (!empty($data['data']['playurl'])) {
            $this->duration = (int)($data['data']['playurl']['duration'] / 1000);
            $this->source = 'bilibili';
            if (!empty($data['data']['playurl']['audio_resource'])) {
                $lastId = count($data['data']['playurl']['audio_resource']) - 1;
                if (!empty($data['data']['playurl']['audio_resource'][$lastId]['backup_url'][0])) {
                    $media = new Media($data['data']['playurl']['audio_resource'][$lastId]['backup_url'][0], '320kbps', 'm4a', false, true);
                    $this->medias[] = $media;
                    $cdnHost = parse_url($this->medias[0]->url, PHP_URL_HOST);
                }
            }
            foreach ($data['data']['playurl']['video'] as $video) {
                if (!empty($foundFormats[$video['stream_info']['desc_words']])) {
                    continue;
                }
                $foundFormats[$video['stream_info']['desc_words']] = true;
                if (!empty($video['video_resource']['backup_url'])) {
                    $media = new Media($video['video_resource']['backup_url'][0], $video['stream_info']['desc_words'], 'mp4', true, false);
                    $this->medias[] = $media;
                } else if (!empty($video['video_resource']['url'])) {
                    $media = new Media($video['video_resource']['url'], $video['stream_info']['desc_words'], 'mp4', true, false);
                    $media->size = $video['video_resource']['size'];
                    $this->medias[] = $media;
                }
            }
            usort($this->medias, array('Helpers', 'sortByQuality'));
        }
    }

    private function estimateVideoSize($bandwidth, $duration)
    {
        return ($duration / 60.0) * ($bandwidth * 10.0);
    }
}