<?php

class Instagram extends Downloader
{
    public function fetch($videoUrl)
    {
        $preferredApi = get_option('aiodl_instagram_api_preference');
        switch ($preferredApi) {
            case 'rocketapi':
                $this->fetch1($videoUrl);
                break;
            case 'aiodown_v1':
                $this->fetchFromApi($videoUrl);
                if (empty($this->medias)) {
                    sleep(5);
                    $this->fetchFromApi($videoUrl);
                }
                break;
            default:
                $this->fetchFromApi2($videoUrl);
                break;
        }
        if (empty($this->thumbnail)) {
            $this->thumbnail = 'https://cdn.nicheoffice.web.tr/image/PGF3juNY8G.jpg';
        }
        if (empty($this->title)) {
            $this->title = 'Instagram Post ' . rand(1, 50);
        }
        $this->encodeThumbnail();
    }

    public function fetchFromApi2($videoUrl)
    {
        $http = new Http('https://api.aiodown.com/v2/instagram/video?postUrl=' . urlencode($videoUrl));
        $http->run();
        $data = json_decode($http->response, true);
        if (!empty($data['error']) && $data['error'] === true) {
            return;
        }
        $this->title = $data['title'] ?? 'Instagram Post ' . rand(1, 50);
        $this->title = Helpers::shortenText($this->title, 128);
        $this->source = 'instagram';
        // $query = parse_url($data['download_url'], PHP_URL_QUERY);
        // parse_str($query, $output);
        switch ($data['type']) {
            case 'video':
                $this->medias[] = new Media($data['download_url'], 'hd', 'mp4', true, true);
                break;
            case 'image':
                $this->medias[] = new Media($data['thumb'], 'hd', 'jpg', false, false);
                break;
            case 'album':
                foreach ($data['medias'] as $media) {
                    if (empty($data['thumb'])) {
                        $data['thumb'] = $media['download_url'];
                    }
                    $ext = $media['type'] === 'video' ? 'mp4' : 'jpg';
                    $this->medias[] = new Media($media['download_url'], 'hd', $ext, false, false);
                }
                break;
        }
        $this->thumbnail = $data['thumb'] ?? null;
    }

    public function fetchFromApi($videoUrl)
    {
        $http = new Http('https://api.aiodown.com/v1/instagram/video?postUrl=' . urlencode($videoUrl));
        $http->run();
        $data = json_decode($http->response, true);
        if (empty($data['medias'])) {
            return;
        }
        $this->title = $data['title'] ?? 'Instagram Post ' . rand(1, 50);
        $this->title = Helpers::shortenText($this->title, 128);
        $this->source = 'instagram';
        $this->thumbnail = null;
        if (!empty($data['duration'])) {
            $this->duration = (int)$data['duration'];
        }
        if (!empty($data['thumbnail'])) {
            $this->thumbnail = $data['thumbnail'];
        }
        foreach ($data['medias'] as $media) {
            if ($media['extension'] == 'jpg') {
                $this->medias[] = new Media($media['url'], 'hd', $media['extension'], false, false);
                if ($this->thumbnail == null) {
                    $this->thumbnail = $media['url'];
                }
                continue;
            }
            $this->medias[] = new Media($media['url'], 'hd', $media['extension'], true, true);
            if ($this->thumbnail == null && !empty($media['thumbnail'])) {
                $this->thumbnail = $media['thumbnail'];
            }
        }
        if ($this->thumbnail == null) {
            $this->thumbnail = 'https://cdn.nicheoffice.web.tr/image/PGF3juNY8G.jpg';
        }
        usort($this->medias, array('Helpers', 'sortByQuality'));
    }

    public function fetch1($videoUrl)
    {
        $rapidApiKey = get_option('aiodl_rapid_api_key');
        $shortcode = $this->getPostShortcode($videoUrl);
        if (!empty($rapidApiKey) && !empty($shortcode)) {
            $hash = sha1($shortcode);
            $cachedData = get_transient($hash);
            if (!empty($cachedData)) {
                $data = $cachedData;
            } else {
                $http = new Http('https://rocketapi-for-instagram.p.rapidapi.com/instagram/media/get_info_by_shortcode');
                $http->addCurlOption(CURLOPT_CUSTOMREQUEST, 'POST');
                $http->addCurlOption(CURLOPT_POSTFIELDS, json_encode(['shortcode' => $shortcode]));
                $http->addHeader('X-RapidAPI-Host', 'rocketapi-for-instagram.p.rapidapi.com');
                $http->addHeader('X-RapidAPI-Key', $rapidApiKey);
                $http->addHeader('Content-Type', 'application/json');
                $http->run();
                $data = json_decode($http->response, true);
                if (!empty($data) && !empty($data['response']) && $data['response']['status_code'] === 200 && !empty($data['response']['body']['items'])) {
                    set_transient($hash, $data, 86400);
                }
            }
            if (!empty($data['response']) && $data['response']['status_code'] === 200 && !empty($data['response']['body']['items'])) {
                $data = $data['response']['body']['items'][0];
                $this->title = $data['caption']['text'] ?? 'Instagram Post';
                if (!empty($data['image_versions2'])) {
                    $this->thumbnail = $data['image_versions2']['candidates'][0]['url'];
                } else if (!empty($data['carousel_media'])) {
                    $this->thumbnail = $data['carousel_media'][0]['image_versions2']['candidates'][0]['url'];
                }
                $this->duration = $data['video_duration'] ?? null;
                $this->source = 'instagram';
                if (!empty($data['video_versions'])) {
                    foreach ($data['video_versions'] as $video) {
                        $this->medias[] = new Media($video['url'], $video['width'] . 'p', 'mp4', true, true);
                    }
                    usort($this->medias, array('Helpers', 'sortByQuality'));
                } else if ($data['media_type'] === 1) {
                    $this->medias[] = new Media($data['image_versions2']['candidates'][0]['url'], 'hd', 'jpg', false, false);
                } else if ($data['media_type'] == 8 && !empty($data['carousel_media'])) {
                    foreach ($data['carousel_media'] as $media) {
                        $this->medias[] = new Media($media['image_versions2']['candidates'][0]['url'], 'hd', 'jpg', false, false);
                    }
                }
            }
        }
    }

    /**
     * @param string $url
     * @return mixed
     */
    private function getPostShortcode($url)
    {
        if (substr($url, -1) != '/') {
            $url .= '/';
        }
        preg_match('/\/(p|tv|reel)\/(.*?)\//', $url, $output);
        return ($output['2'] ?? '');
    }
}