<?php

class Mixcloud extends Downloader
{
    public function fetch($videoUrl)
    {
        $explode = explode('/', $videoUrl);
        if (empty($explode[3]) || empty($explode[4])) {
            return;
        }
        $username = $explode[3];
        $slug = $explode[4];
        $http = new Http('https://app.mixcloud.com/graphql?query=%7B%0A++cloudcastLookup%28lookup%3A+%7Busername%3A+%22' . $username . '%22%2C+slug%3A+%22' . $slug . '%22%7D%29+%7B%0A++++audioLength%0A++++comments%28first%3A+100%29+%7B%0A++++++edges+%7B%0A++++++++node+%7B%0A++++++++++comment%0A++++++++++created%0A++++++++++user+%7B%0A++++++++++++displayName%0A++++++++++++username%0A++++++++++%7D%0A++++++++%7D%0A++++++%7D%0A++++++totalCount%0A++++%7D%0A++++description%0A++++favorites+%7B%0A++++++totalCount%0A++++%7D%0A++++featuringArtistList%0A++++isExclusive%0A++++name%0A++++owner+%7B%0A++++++displayName%0A++++++url%0A++++++username%0A++++%7D%0A++++picture%28width%3A+1024%2C+height%3A+1024%29+%7B%0A++++++++url%0A++++%7D%0A++++plays%0A++++publishDate%0A++++reposts+%7B%0A++++++totalCount%0A++++%7D%0A++++streamInfo+%7B%0A++++++dashUrl%0A++++++hlsUrl%0A++++++url%0A++++%7D%0A++++tags+%7B%0A++++++tag+%7B%0A++++++++name%0A++++++%7D%0A++++%7D%0A++++restrictedReason%0A++++id%0A++%7D%0A%7D');
        $http->run();
        $data = json_decode($http->response, true);
        if (empty($data['data']['cloudcastLookup']['streamInfo']['dashUrl'])) {
            $this->fetchLegacy($videoUrl);
            return;
        }
        $this->title = $data['data']['cloudcastLookup']['name'];
        $this->source = 'mixcloud';
        $this->thumbnail = $data['data']['cloudcastLookup']['picture']['url'];
        $this->duration = $data['data']['cloudcastLookup']['audioLength'];
        $mediaUrl = $this->decryptXORCipher($data['data']['cloudcastLookup']['streamInfo']['dashUrl']);
        $http = new Http($mediaUrl);
        $http->run();
        $chunks = $this->parseMPD($http->response);
        if (empty($chunks)) {
            return;
        }
        $chunkSize = new Http($chunks[1]);
        $chunkSize = $chunkSize->getFileSize();
        $chunkFile = new Cache('mixcloud-' . $username . '-' . $slug, 'json', json_encode($chunks));
        $media = new Media($chunkFile->url, '128kbps', 'webm', false, true);
        $media->size = $chunkSize * count($chunks);
        $media->chunked = true;
        $this->medias[] = $media;
    }

    private function decryptXORCipher($ciphertext)
    {
        $http = new Http('https://api.aiodown.com/v1/mixcloud/decrypt?text=' . urlencode($ciphertext));
        $http->run();
        return $http->response;
    }

    private function parseMPD($xmlString)
    {
        if (empty($xmlString)) {
            return [];
        }
        $xml = simplexml_load_string($xmlString);
        $initUrl = (string)$xml->Period->AdaptationSet->SegmentTemplate->attributes()['initialization'];
        $representationId = (string)$xml->Period->AdaptationSet->Representation->attributes()['id'];
        $audioServerUrl = parse_url((string)$xml->Period->AdaptationSet->SegmentTemplate->attributes()['media'], PHP_URL_HOST);
        $segments = $xml->Period->AdaptationSet->SegmentTemplate->SegmentTimeline->S;
        $fragmentURLs = [];
        $fragmentURLs[] = str_replace('$RepresentationID$', $representationId, $initUrl);
        $fragmentNumber = 1;
        foreach ($segments as $segment) {
            $segmentDuration = (int)$segment->attributes()['d'];
            $repeatCount = isset($segment->attributes()['r']) ? (int)$segment->attributes()['r'] : 1;
            for ($i = 0; $i < $repeatCount; $i++) {
                $fragmentURL = (string)$xml->Period->AdaptationSet->SegmentTemplate->attributes()['media'];
                $fragmentURL = str_replace('$Number$', $fragmentNumber, $fragmentURL);
                $fragmentURL = str_replace('$RepresentationID$', $representationId, $fragmentURL);
                $fragmentURLs[] = $fragmentURL;
                $fragmentNumber += 1;
            }
        }
        return $fragmentURLs;
    }
}